@@ -26,28 +26,6 @@
 #include "mdadm.h"
 #include "md_p.h"
 
-/* from readme.c */
-mapping_t pers[] = {
-	{ "linear", LEVEL_LINEAR},
-	{ "raid0", 0},
-	{ "0", 0},
-	{ "stripe", 0},
-	{ "raid1", 1},
-	{ "1", 1},
-	{ "mirror", 1},
-	{ "raid4", 4},
-	{ "4", 4},
-	{ "raid5", 5},
-	{ "5", 5},
-	{ "multipath", LEVEL_MULTIPATH},
-	{ "mp", LEVEL_MULTIPATH},
-	{ "raid6", 6},
-	{ "6", 6},
-	{ "raid10", 10},
-	{ "10", 10},
-	{ NULL, 0}
-};
-
 #ifndef MDASSEMBLE_AUTO
 /* from mdopen.c */
 int open_mddev(char *dev, int report_errors/*unused*/)
